import datetime
import json
from django.utils.html import escape

# fibonacci numbers
# (extend this list when there are more C3s)
fib = [0.5, 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144, 233, 377, 610, 987, 1597, 2584, 4181, 6765, 10946, 17711, 28657,
       46368, 75025, 121393, 196418, 317811, 514229, 832040, 1346269, 2178309, 3524578, 5702887, 9227465]


class HashableDict(dict):
    def __hash__(self):
        return hash(tuple(sorted(self.items())))


def participation_score(name):
    score = 0
    for i in range(len(c3s)):
        c3 = c3s[i]
        if name in map(lambda x: x['authors'][0]['name'], c3['threads']):
            score += fib[i]
    return score


def participation_by_user(name, binary=False):
    if binary:
        participation = ''.join(
            ['1' if name in map(lambda x: x['authors'][0]['name'], c3['threads']) else '0' for c3 in c3s])
    else:
        participation = [c3['name'] for c3 in c3s if name in map(lambda x: x['authors'][0]['name'], c3['threads'])]

    return participation


if __name__ == '__main__':

    current_year = datetime.date.today().year

    with open('c3s.json', encoding='utf-8') as f:
        c3s = json.load(f)

    users = sorted(list(set([HashableDict(thread['authors'][0]) for c3 in c3s for thread in c3['threads']])),
                   key=lambda u: u['name'])
    users.sort(
        key=lambda u: participation_score(u['name']),
        reverse=True)

    with open('c3stats.html', 'w', encoding='utf-8') as f:

        f.write(f'<title>C3 participants, {current_year} edition</title>')
        f.write(f'<h1>C3 participants, {current_year} edition</h1>')
        f.write('<p>Hover over a number to see the thread title.</p>')
        f.write('<table border="1" cellpadding="4">')
        f.write('<thead>')
        f.write('<tr>')
        f.write('<th>User</th>')
        for c3 in c3s:
            f.write(
                f'<th><a href="https://www.smwcentral.net/?p=viewforum&f={c3["forum_id"]}">{escape(c3["name"])}</a></th>')
        f.write('</tr>')
        f.write('</thead>')
        f.write('<tbody>')
        for user in users:
            c3names = participation_by_user(user['name'])
            f.write('<tr>')
            f.write(
                f'<td><a href="https://www.smwcentral.net/?p=profile&id={user["id"]}">{escape(user["name"])}</a></td>')
            for c3 in c3s:
                f.write('<td>')
                if c3['name'] in c3names:
                    threads_by_user = sorted([thread for thread in c3['threads'] for author in thread['authors'] if
                                              author['id'] == user['id']], key=lambda t: t['id'])
                    for i in range(len(threads_by_user)):
                        thread = threads_by_user[i]
                        f.write(
                            f'<a href="https://www.smwcentral.net/?p=viewthread&t={thread["id"]}" title="{escape(thread["name"])}">{i + 1}</a> ')
                f.write('</td>')
            f.write('</tr>')
        f.write('</tbody>')
        f.write('</table>')
